import logging

from flask import Blueprint, request
from flask_jwt_extended import jwt_required
from flask_restful import Api, Resource

from portal.service import general, auth
from portal.service.groups import delete_group, update_group, create_group, \
    update_user_group_membership, find_users_in_group_by_role, add_single_user_to_group, \
    remove_single_user_from_group, import_group, list_groups, remove_project_from_group, \
    add_project_to_group, find_projects_in_group
from portal.service.permissions import check_client
from portal.tools.decorators import error_handler
from portal.rest import rest_helpers
from portal.rest.schemas import users_schema, group_schema, groups_schema, user_list_update_schema, \
    group_import_schema
from portal.service.errors import ForbiddenError

groups_blueprint = Blueprint('groups', __name__)
groups_api = Api(groups_blueprint)

log = logging.getLogger(__name__)


class GroupResource(Resource):
    @error_handler
    @jwt_required
    def get(self, cid, gid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        permissions = ['view_course_full', 'view_course_limited']
        if not check_client(client=client, course=course, permissions=permissions):
            raise ForbiddenError(uid=client.id)

        group = general.find_group(course, gid)
        return group_schema.dump(group)

    @error_handler
    @jwt_required
    def delete(self, cid, gid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        if not check_client(client=client, course=course, permissions=['update_course']):
            raise ForbiddenError(uid=client.id)

        group = general.find_group(course, gid)
        delete_group(group)
        return '', 204

    @error_handler
    @jwt_required
    def put(self, cid, gid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        permissions = ['write_groups', 'update_course']
        if not check_client(client=client, course=course, permissions=permissions):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            group_schema, action='update', resource='group'
        )

        group = general.find_group(course, gid)
        update_group(group, data)
        return '', 204


class GroupsList(Resource):
    @error_handler
    @jwt_required
    def get(self, cid):
        client = auth.find_client()
        course = general.find_course(cid)

        return groups_schema.dump(list_groups(course, client=client))

    @error_handler
    @jwt_required
    def post(self, cid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        if not check_client(client=client, course=course, permissions=['update_course']):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            group_schema, action='create', resource='group'
        )

        new_group = create_group(course, **data)
        return group_schema.dump(new_group)[0], 201


class GroupUsersList(Resource):
    @error_handler
    @jwt_required
    def get(self, cid, gid):
        client = auth.find_client()
        course = general.find_course(cid)
        role_id = request.args.get('role')

        group = general.find_group(course, gid)
        # authorization
        if not check_client(client=client, course=course, permissions='view_course_full') or \
                (check_client(client=client, course=course, permissions=['view_course_limited'])
                 and client in group.users):
            raise ForbiddenError(uid=client.id)

        users = find_users_in_group_by_role(course, group, role_id)
        return users_schema.dump(users)

    @error_handler
    @jwt_required
    def put(self, cid, gid):
        client = auth.find_client()
        course = general.find_course(cid)
        # authorization
        permissions = ['write_groups', 'update_course']
        if not check_client(client=client, course=course, permissions=permissions):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            user_list_update_schema, action='update', resource='group_membership'
        )

        group = general.find_group(course, gid)
        # everything from users_add is added, THEN everything from users_remove is removed
        update_user_group_membership(group, data)
        return '', 204


class GroupUser(Resource):
    @error_handler
    @jwt_required
    def put(self, cid, gid, uid):
        client = auth.find_client()
        course = general.find_course(cid)

        # authorization
        permissions = ['write_groups', 'update_course']
        if not check_client(client=client, course=course, permissions=permissions):
            raise ForbiddenError(uid=client.id)

        group = general.find_group(course, gid)
        user = general.find_user(uid)
        add_single_user_to_group(group, user)
        return '', 204

    @error_handler
    @jwt_required
    def delete(self, cid, gid, uid):
        client = auth.find_client()
        course = general.find_course(cid)

        # authorization
        if not check_client(client=client, course=course, permissions=['write_groups', 'update_course']):
            raise ForbiddenError(uid=client.id)

        group = general.find_group(course, gid)
        user = general.find_user(uid)
        remove_single_user_from_group(group, user)
        return '', 204


class GroupProjectsList(Resource):
    @error_handler
    @jwt_required
    def get(self, cid, gid):
        client = auth.find_client()
        course = general.find_course(cid)
        group = general.find_group(course, gid)
        # authorization
        if not check_client(client=client, course=course, permissions=['view_course_full']) or\
                (check_client(client=client, course=course, permissions=['view_course_limited'])
                 and client in group.users):
            raise ForbiddenError(uid=client.id)

        users = find_projects_in_group(group)
        return users_schema.dump(users)


class GroupProject(Resource):
    @error_handler
    @jwt_required
    def put(self, cid, gid, pid):
        client = auth.find_client()
        course = general.find_course(cid)

        # authorization
        permissions = ['write_groups', 'update_course']
        if not check_client(client=client, course=course, permissions=permissions):
            raise ForbiddenError(uid=client.id)

        group = general.find_group(course, gid)
        project = general.find_project(course, pid)
        add_project_to_group(group, project)
        return '', 204

    @error_handler
    @jwt_required
    def delete(self, cid, gid, pid):
        client = auth.find_client()
        course = general.find_course(cid)

        # authorization
        if not check_client(client=client, course=course, permissions=['write_groups', 'update_course']):
            raise ForbiddenError(uid=client.id)

        group = general.find_group(course, gid)
        project = general.find_project(course, pid)
        remove_project_from_group(group, project)
        return '', 204


class GroupImport(Resource):
    @error_handler
    @jwt_required
    def put(self, cid):
        client = auth.find_client()
        target_course = general.find_course(cid)
        # authorization
        permissions = ['write_groups', 'update_course']
        if not check_client(client=client, course=target_course, permissions=permissions):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(
            group_import_schema, action='import', resource='group'
        )
        new_group = import_group(data, target_course)
        return group_schema.dump(new_group), 201


groups_api.add_resource(GroupResource, '/<string:gid>')
groups_api.add_resource(GroupsList, '')
groups_api.add_resource(GroupUsersList, '/<string:gid>/users')
groups_api.add_resource(GroupUser, '/<string:gid>/users/<string:uid>')
groups_api.add_resource(GroupProject, '/<string:gid>/projects/<string:pid>')
groups_api.add_resource(GroupProjectsList, '/<string:gid>/projects')
groups_api.add_resource(GroupImport, '/import')
