import logging

from flask import Blueprint
from flask_jwt_extended import jwt_required
from flask_restful import Api, Resource

import portal.service.auth
import portal.service.general
from portal.service.reviews import create_review_items, create_review
from portal.rest import rest_helpers
from portal.rest.schemas import submission_schema, submission_state_schema, review_schema
from portal.service.permissions import check_client
from portal.service.submissions import delete_submission, \
    update_submission_state, copy_submission
from portal.tools.decorators import error_handler
from portal.service.errors import ForbiddenError
from portal.service import permissions, auth, general

submissions_blueprint = Blueprint('submissions', __name__)
submissions_api = Api(submissions_blueprint)

log = logging.getLogger(__name__)


def submission_access_group(client, submission, course, perm):
    if permissions.check_client(client, course, perm):
        group_intersection = [group for group in client.groups if submission.user in group]

        return any(group in submission.project.groups for group in group_intersection)
    return False


class SubmissionResource(Resource):
    @error_handler
    @jwt_required
    def get(self, sid):
        client = portal.service.auth.find_client()
        submission = portal.service.general.find_submission(sid)
        course = submission.project.course
        # authorization
        if not permissions.check_client(client, course, ['read_submissions_all']) or \
                submission_access_group(client, submission, course, ['read_submissions_groups']) or \
                (permissions.check_client(client, course, ['read_submissions_own']) and submission.user == client):
            raise ForbiddenError(uid=client.id)
        return submission_schema.dump(submission)

    @error_handler
    @jwt_required
    def delete(self, sid):
        client = auth.find_client()
        # authorization
        if not permissions.check_sysadmin(client):
            raise ForbiddenError(uid=client)

        submission = general.find_submission(sid)
        delete_submission(submission)
        return '', 204


class SubmissionState(Resource):
    @error_handler
    @jwt_required
    def get(self, sid):
        client = auth.find_client()
        submission = general.find_submission(sid)
        course = submission.project.course
        # authorization
        if not permissions.check_client(client, course, ['read_submissions_all']) or \
                submission_access_group(client, submission, course, ['read_submissions_groups']) or \
                (permissions.check_client(client, course, ['read_submissions_own']) and submission.user == client):
            raise ForbiddenError(uid=client.id)

        return submission_state_schema.dump(submission)[0], 200

    @error_handler
    @jwt_required
    def put(self, sid):
        client = auth.find_client()
        submission = general.find_submission(sid)
        # authorization
        if not (permissions.check_component(component=client) or
                permissions.check_sysadmin(client) or
                client == submission.user):
            raise ForbiddenError(uid=client)

        data = rest_helpers.parse_request_data(
            submission_state_schema, action='update_state', resource='submission'
        )
        update_submission_state(client, submission, data)
        return '', 204


class SubmissionResult(Resource):
    @error_handler
    @jwt_required
    def put(self, sid):
        client = auth.find_client()
        submission = general.find_submission(sid)
        # authorization
        return {"message": "Not implemented."}, 200


class SubmissionFiles(Resource):
    @error_handler
    @jwt_required
    def get(self, sid):
        client = auth.find_client()
        # the user wants a zip or a json
        # authorization
        return {"message": "Not implemented."}, 200


class SubmissionSourceFiles(Resource):
    @error_handler
    @jwt_required
    def get(self, sid):
        client = auth.find_client()
        return {"message": "Not implemented."}, 200


class SubmissionTestFiles(Resource):
    @error_handler
    @jwt_required
    def get(self, sid):
        client = auth.find_client()
        return {"message": "Not implemented."}, 200


class SubmissionResultFiles(Resource):
    @error_handler
    @jwt_required
    def get(self, sid):
        client = auth.find_client()
        return {"message": "Not implemented."}, 200


class SubmissionResubmit(Resource):
    @error_handler
    @jwt_required
    def post(self, sid):
        client = auth.find_client()
        source_submission = general.find_submission(sid)
        # authorization
        course = source_submission.project.course
        if not check_client(client=client, course=course, permissions=['resubmit_submissions']):
            raise ForbiddenError(uid=client)

        data = rest_helpers.parse_request_data(
            submission_schema, action='resubmit', resource='submission'
        )
        # create new submission by copying files from the source submission in storage
        new_submission = copy_submission(source_submission, note=data['note'])
        return submission_schema.dump(new_submission), 201


class SubmissionReview(Resource):
    @error_handler
    @jwt_required
    def get(self, sid):
        client = auth.find_client()
        submission = general.find_submission(sid)
        course = submission.project.course
        # authorization
        if not permissions.check_client(client, course, ['read_submissions_all']) or \
                submission_access_group(client, submission, course, ['read_submissions_groups']) or \
                (permissions.check_client(client, course, ['read_submissions_own']) and submission.user == client):
            raise ForbiddenError(uid=client.id)

        return review_schema.dump(submission.review)[0], 200

    @error_handler
    @jwt_required
    def post(self, sid):
        client = auth.find_client()
        submission = general.find_submission(sid)
        course = submission.project.course
        # authorization

        if not permissions.check_client(client, course, ['write_reviews_all']) or \
                submission_access_group(client, submission, course, ['write_reviews_group']) or \
                (permissions.check_client(client, course, ['write_reviews_own']) and submission.user == client):
            raise ForbiddenError(uid=client.id)

        data = rest_helpers.parse_request_data(review_schema, action='create', resource='review')

        if not submission.review:
            create_review(submission)

        # also writes to db
        create_review_items(
            review=submission.review,
            items=data['review_items'],
            author=client
        )

        return review_schema.dump(submission.review)[0], 201


submissions_api.add_resource(SubmissionResource, '/<string:sid>')
submissions_api.add_resource(SubmissionState, '/<string:sid>/state')
submissions_api.add_resource(SubmissionResult, '/<string:sid>/result')
submissions_api.add_resource(SubmissionResubmit, '/<string:sid>/resubmit')

submissions_api.add_resource(SubmissionFiles, '/<string:sid>/files')
submissions_api.add_resource(SubmissionSourceFiles, '/<string:sid>/files/sources')
submissions_api.add_resource(SubmissionTestFiles, '/<string:sid>/files/test_files')
submissions_api.add_resource(SubmissionResultFiles, '/<string:sid>/files/results')

submissions_api.add_resource(SubmissionReview, '/<string:sid>/review')
