from portal.database.models import Course, Role, User
from . import utils
import json


def test_list(client):
    cpp = Course.query.filter_by(codename="TestCourse2").first()
    cpp_roles = len(cpp.roles)
    response = utils.make_request(client, f'/courses/{cpp.codename}/roles', method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    roles = utils.extract_data(response)
    assert len(roles) == cpp_roles
    cpp_updated = Course.query.filter_by(codename="TestCourse1").first()
    for r in roles:
        utils.assert_role_in(cpp_updated.roles, r)


def test_create(client):
    cpp = Course.query.filter_by(codename="TestCourse1").first()
    cpp_roles = len(cpp.roles)

    request_dict = {
        "name": "new role",
        "description": "new role desc"
    }
    request_json = json.dumps(request_dict)
    response = utils.make_request(client, f"/courses/{cpp.codename}/roles", data=request_json,
                                  headers={"content-type": "application/json"}, method='post')
    assert response.status_code == 201
    assert response.mimetype == 'application/json'

    new_role = utils.extract_data(response)
    cpp_updated = Course.query.filter_by(codename="TestCourse1").first()
    assert len(cpp_updated.roles) == cpp_roles + 1
    assert new_role['name'] == "new role"
    assert new_role['description'] == "new role desc"
    assert new_role['id']
    utils.assert_role_in(cpp_updated.roles, new_role)


def test_read(client):
    cpp = Course.query.filter_by(codename="TestCourse1").first()
    r = cpp.roles[0]
    response = utils.make_request(client, f"/courses/{cpp.codename}/roles/{r.name}", method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    role = utils.extract_data(response)

    utils.assert_role(r, role)


def test_update(client):
    cpp = Course.query.filter_by(codename="TestCourse1").first()
    r = cpp.roles[0]
    r_name = r.name
    request_dict = dict(
        name="new role name",
        description="new role desc",
    )
    request_json = json.dumps(request_dict)
    response = utils.make_request(client, f"/courses/{cpp.codename}/roles/{r.name}", data=request_json,
                                  headers={"content-type": "application/json"}, method='put')
    assert response.status_code == 204
    assert response.mimetype == 'application/json'
    r_updated = Role.query.filter(Role.course_id == cpp.id).filter(Role.name == "new role name").first()
    assert r_updated
    assert r_updated.description == "new role desc"
    assert not Role.query.filter(Role.course_id == cpp.id).filter(Role.name == r_name).first()


def test_delete(client):
    cpp = Course.query.filter_by(codename="TestCourse1").first()
    r = cpp.roles[0]
    r_name = r.name
    cpp_roles = len(cpp.roles)
    response = utils.make_request(client, f"/courses/{cpp.codename}/roles/{r.name}", method='delete')

    assert response.status_code == 204
    assert response.mimetype == 'application/json'
    cpp_updated = Course.query.filter_by(codename="TestCourse1").first()
    assert len(cpp_updated.roles) == cpp_roles - 1
    assert not Role.query.filter(Role.course_id == cpp.id).filter(Role.name == r_name).first()


def test_users_list(client):
    cpp = Course.query.filter_by(codename="TestCourse1").first()
    r = cpp.roles[0]
    response = utils.make_request(client, f"/courses/{cpp.codename}/roles/{r.id}/users", method='get')

    assert response.status_code == 200
    assert response.mimetype == 'application/json'
    users = utils.extract_data(response)
    assert len(users) == len(r.users)
    for user in users:
        utils.assert_user_in(r.users, user)


def test_users_update_add(client):
    c = Course.query.filter_by(codename="TestCourse2").first()
    roles = [role for role in c.roles if role.name == "student"]
    r = roles[0]
    r_users = len(r.users)
    user = User.query.filter_by(username="student1").first()
    assert user not in r.users

    request_dict = dict(
        add=[user.id]
    )
    request_json = json.dumps(request_dict)
    response = utils.make_request(client, f"/courses/{c.codename}/roles/{r.name}/users", data=request_json,
                                  headers={"content-type": "application/json"}, method='put')

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == c.id).filter(Role.id == r.id).first()
    assert len(r_updated.users) == r_users + 1
    assert user in r_updated.users


def test_users_update_add_duplicate(client):
    c = Course.query.filter_by(codename="TestCourse2").first()
    roles = [role for role in c.roles if role.name == "student"]
    r = roles[0]
    r_users = len(r.users)
    user = r.users[0]
    assert user in r.users

    request_dict = dict(
        add=[user.id]
    )
    request_json = json.dumps(request_dict)
    response = utils.make_request(client, f"/courses/{c.codename}/roles/{r.name}/users", data=request_json,
                                  headers={"content-type": "application/json"}, method='put')

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == c.id).filter(Role.id == r.id).first()
    assert len(r_updated.users) == r_users
    assert user in r_updated.users


def test_users_update_remove(client):
    c = Course.query.filter_by(codename="TestCourse2").first()
    roles = [role for role in c.roles if role.name == "student"]
    r = roles[0]
    r_users = len(r.users)
    user = r.users[0]
    assert user in r.users

    request_dict = dict(
        remove=[user.id]
    )
    request_json = json.dumps(request_dict)
    response = utils.make_request(client, f"/courses/{c.codename}/roles/{r.name}/users", data=request_json,
                                  headers={"content-type": "application/json"}, method='put')

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == c.id).filter(Role.id == r.id).first()
    assert len(r_updated.users) == r_users - 1
    assert user not in r_updated.users


def test_users_update_remove_user_not_in(client):
    c = Course.query.filter_by(codename="TestCourse2").first()
    roles = [role for role in c.roles if role.name == "student"]
    r = roles[0]
    r_users = len(r.users)
    user = User.query.filter_by(username="student1").first()
    assert user not in r.users

    request_dict = dict(
        remove=[user.id]
    )
    request_json = json.dumps(request_dict)
    response = utils.make_request(client, f"/courses/{c.codename}/roles/{r.name}/users", data=request_json,
                                  headers={"content-type": "application/json"}, method='put')

    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == c.id).filter(Role.id == r.id).first()
    assert len(r_updated.users) == r_users
    assert user not in r_updated.users


def test_add_user(client):
    c = Course.query.filter_by(codename="TestCourse2").first()
    roles = [role for role in c.roles if role.name == "student"]
    r = roles[0]
    r_users = len(r.users)

    u = User.query.filter_by(username="student1").first()
    assert r not in u.roles

    response = utils.make_request(client, f"/courses/{c.codename}/roles/{r.name}/users/{u.username}", method='put')
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == c.id).filter(Role.id == r.id).first()
    assert u in r_updated.users
    assert len(r_updated.users) == r_users + 1


def test_add_user_duplicate(client):
    c = Course.query.filter_by(codename="TestCourse2").first()
    roles = [role for role in c.roles if role.name == "student"]
    r = roles[0]
    r_users = len(r.users)
    u = r.users[0]
    assert u in r.users

    response = utils.make_request(client, f"/courses/{c.codename}/roles/{r.name}/users/{u.username}", method='put')
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == c.id).filter(Role.id == r.id).first()
    assert u in r_updated.users
    assert len(r_updated.users) == r_users


def test_remove_user(client):
    c = Course.query.filter_by(codename="TestCourse2").first()
    roles = [role for role in c.roles if role.name == "student"]
    r = roles[0]
    r_users = len(r.users)
    u = r.users[0]

    response = utils.make_request(client, f"/courses/{c.codename}/roles/{r.name}/users/{u.username}", method='delete')
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == c.id).filter(Role.id == r.id).first()
    assert u not in r_updated.users
    assert len(r_updated.users) == r_users - 1


def test_remove_user_not_in(client):
    c = Course.query.filter_by(codename="TestCourse2").first()
    roles = [role for role in c.roles if role.name == "student"]
    r = roles[0]
    r_users = len(r.users)
    u = User.query.filter_by(username="student1").first()
    assert r not in u.roles

    response = utils.make_request(client, f"/courses/{c.codename}/roles/{r.name}/users/{u.username}", method='delete')
    assert response.status_code == 400
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == c.id).filter(Role.id == r.id).first()
    assert u not in r_updated.users
    assert len(r_updated.users) == r_users


def test_permissions_list(client):
    cpp = Course.query.filter_by(codename="TestCourse1").first()
    r = cpp.roles[0]

    response = utils.make_request(client, f"/courses/{cpp.codename}/roles/{r.name}/permissions", method='get')
    assert response.status_code == 200
    assert response.mimetype == 'application/json'

    data = utils.extract_data(response)
    utils.assert_role_permissions(r.permissions, data)


def test_permissions_update(client):
    cpp = Course.query.filter_by(codename="TestCourse1").first()
    r = cpp.roles[0]
    r_name = r.name

    request_dict = dict(
        view_course_full=True,
        write_roles=True,
        read_submissions_all=True
    )
    request_json = json.dumps(request_dict, cls=utils.DateTimeEncoder)

    response = utils.make_request(client, f"/courses/{cpp.codename}/roles/{r.name}/permissions", data=request_json,
                                  headers={"content-type": "application/json"}, method='put')
    assert response.status_code == 204
    assert response.mimetype == 'application/json'

    r_updated = Role.query.filter(Role.course_id == cpp.id).filter_by(name=r_name).first()
    assert r_updated.permissions.view_course_full
    assert r_updated.permissions.write_roles
    assert r_updated.permissions.read_submissions_all
